 
/** 
 * Empresa = Nome (String) + Stock = Map(Codigo -> Produto)
 * 
 * @author F. Mrio Martins
 * @version 1.0/01/2005
 */
import java.util.*;
public class Empresa {

    // Variveis de Instncia   
    private String empresa;                   // Nome da Empresa
    private TreeMap<String, Produto> stock;   // Cdigo -> Produto
		
	// Construtores bsicos 
	public Empresa() { empresa = ""; stock = new TreeMap<String, Produto>(); }
	
	public Empresa(String nome) { 
	    empresa = nome; stock = new TreeMap<String, Produto>();
	}
	
	public Empresa(Empresa emp) {
	    empresa = emp.getNome();
	    stock = new TreeMap<String, Produto>();
	    for(Produto p : emp.getStock().values())
	        stock.put(p.getCodigo(), p.clone());
	}	    
	
	public Empresa(String nomeEmp, Map<String, Produto> stockInic) {
	    empresa = nomeEmp;
	    stock = new TreeMap<String, Produto>();
	    for(Produto p : stockInic.values())
	        stock.put(p.getCodigo(), p.clone());
	}
		
	// Mtodos de Instncia
	
   /** Devolve o nome da empresa */ 
   public String getNome() { return empresa; }
   
   /** Devolve uma cpia do stock actual da empresa */
   public Map<String, Produto> getStock() {
       TreeMap<String, Produto> stk = new TreeMap<String, Produto>();
       for(Produto p : stock.values()) stk.put(p.getCodigo(), p.clone());
       return stk;
   }
   
   /**
    * Devolve todos os cdigos vlidos de produtos
    */
    public TreeSet<String> codigos() {	    
	  TreeSet<String> codigos = new TreeSet<String>();   
	  for(Produto p : stock.values()) codigos.add(p.getCodigo());	          
	  return codigos;
    }
   
   /**
	 *  Insere um novo produto. O mtodo  consistente dado 
	 *  que o cdigo do produto  igual ao que consta
	 *  da ficha do produto !!
	 */
    public void insereProduto(Produto fichaProd) {		
		stock.put(fichaProd.getCodigo(), fichaProd.clone());
    }

    /** Remove o produto cujo cdigo  dado como parmetro */ 
	public void removeProduto(String cod) { stock.remove(cod); }
	
	/** Devolve o nmero actual de produtos em stock */
    public int numProdutos() { return stock.size(); }
		  
    /** Verifica se um dado produto cujo cdigo  dado existe no stock. */	
	public boolean existeProd(String cod) {
	    return stock.keySet().contains(cod);
	}
	
	/**
	 * Abate uma dada quantidade  existncia em stock de um produto cujo 
	 * cdigo  dado.  uma alterao => GET -> CHANGE -> REMOVE -> PUT
	 */
	public void abateQuantStock(String cod, int quant) {
	    Produto prod = new Produto(); prod = stock.get(cod);
	    prod.saidaStock(quant); 
	}
		   
    /**
	 *  Cria a lista com os cdigos dos produtos que possuem quantidade em ALARME 
	 */
	public List<String> prodsEmAlarme() {	    
	  ArrayList<String> codigos = new ArrayList<String>();   
	  for(Produto p : stock.values())
	       if( p.estaEmAlarme() ) codigos.add(p.getCodigo()) ;	         
	  return codigos;
    }
   
    /** Conjunto dos cdigos dos produtos que possuem quantidade no em ALARME */
	public Set<String> prodsOk() {	    
	  TreeSet<String> codigos = new TreeSet<String>();   
	  for(Produto p : stock.values())
	          if( p.estaEmAlarme() ) codigos.add(p.getCodigo()) ;	         
	 return codigos;
    }
       
   /** toString() */
    public String toString() {
        StringBuilder s = new StringBuilder();
        s.append("EMPRESA : "); s.append(empresa + "\n");
        s.append("-----------  STOCK --------------------\n");
        for(Produto p : stock.values()) {
	         s.append(p.toString()); 
	         s.append("--------------------------\n");
	   }
	   return s.toString();
    }   
    
    
    /**
     * Devolve toda a informao de um produto cujo
     * cdigo  dado e garantidamente existente
     */
    public Produto fichaProduto(String cod) {
        return stock.get(cod).clone();
    }
    
    /** Quantidade total de produtos em stock */
    public int quantTotalEmStock() {	    
	   int total = 0;
	   for(Produto p : stock.values()) total += p.getQuant();	          
	   return total;
    }
    
    /** clone */
    public Empresa clone() { return new Empresa(this); }
     
 }




	
	